
// ==============================================================================================================================
//    Fresh!mindworkz Intro Skeleton - Import MXM Player Functions.
// ------------------------------------------------------------------------------------------------------------------------------
//    Copyright (C) 2002. Pasy & Pontscho & Remage / Fresh!mindworkz.
// ==============================================================================================================================

{$LINKLIB MxmPlay.obj}

function xmpInit: dword; external name 'xmpInit';
procedure xmpPlay; external name 'xmpPlay';
procedure xmpRender; external name 'xmpRender';
procedure xmpSetVolume; external name 'xmpSetVolume';
function xmpGetSync: byte; external name 'xmpGetSync';
function xmpGetPos: word; external name 'xmpGetPos';


{$DEFINE FIS_PENTIUM_PRO_OPTIMIZATION}

const

  // ----- Buffer Size (Should Be PowerOf2...). Latency: 65536 / 44100 Hz = 1.486 Seconds.
  FIS_WAVEOUT_BUFFERSIZE = 65536;

  // ----- WaveOut Thread Priority.
  FIS_WAVEOUT_PRIORITY = THREAD_PRIORITY_TIME_CRITICAL;


// ------------------------------------------------------------------------------------------------------------------------------
//    Initialized Variables.
// ------------------------------------------------------------------------------------------------------------------------------

const

  fisWaveOut: HWAVEOUT = 0;
  fisWaveFormatEx: WAVEFORMATEX =
    (
      wFormatTag:       WAVE_FORMAT_PCM;
      nChannels:        2;
      nSamplesPerSec:   44100;
      nAvgBytesPerSec:  44100*2*2;
      nBlockAlign:      4;
      wBitsPerSample:   16;
      cbSize:           0;
    );

  fisWaveHdr: WAVEHDR =
    (
      lpData:           nil;
      dwBufferLength:   FIS_WAVEOUT_BUFFERSIZE * 4;
      dwBytesRecorded:  0;
      dwUser:           0;
      dwFlags:          WHDR_BEGINLOOP or WHDR_ENDLOOP;
      dwLoops:          -1;
      // ...
    );

  fisWaveOutBuffer: ^smallint = nil;

  fisMMTime: MMTIME =
    (
      wType:  TIME_SAMPLES;
      // ...
    );

  fisMxmThreadID: longint = 0;
  fisMxmThread: HANDLE = 0;

  fisMxmThreadExitRequest: boolean = true;

  fisMxmBuffer: ^dword = nil;
  fisMxmHeap: ^byte = nil;


// ------------------------------------------------------------------------------------------------------------------------------
//    function fisMxmThreadProc()
// ------------------------------------------------------------------------------------------------------------------------------

function fisMxmThreadProc( _Parameter: pointer ): dword; stdcall;
var
  _Position, _PrevPosition, _Length: dword;
  _Result: MMRESULT;
begin

  fisMxmThreadProc := 0;

  _PrevPosition := 0;
  _Position := 0;

  while ( not fisMxmThreadExitRequest ) do
    begin

      // ----- Get Current Playback Position.

      _Result := waveOutGetPosition( fisWaveOut, @fisMMTime, sizeof( MMTIME ));
      if ( _Result <> MMSYSERR_NOERROR ) then
        Exit;

      // ----- Calculate How Many Bytes To Render.

      _PrevPosition := _Position and ( FIS_WAVEOUT_BUFFERSIZE-1 );
      _Position := fisMMTime.Sample and ( FIS_WAVEOUT_BUFFERSIZE-1 );

      if ( _Position <> _PrevPosition ) then
        begin

          if ( _Position < _PrevPosition ) then
            _Position := FIS_WAVEOUT_BUFFERSIZE;

          _Length := _Position - _PrevPosition;

          // ----- Render Wave & Convert To 16bit PCM Format.

          if ( _Length > 0 ) then
          asm
            pushad

            cld
            mov   eax, [ _Length ]
            mov   edi, [ fisMxmBuffer ]
            call  xmpRender

            mov   esi, dword [ fisMxmBuffer ]
            mov   edi, dword [ _PrevPosition ]
            shl   edi, 2
            add   edi, dword [ fisWaveOutBuffer ]
            mov   ecx, dword [ _Length ]
            shl   ecx, 1
            mov   ebx, 32700
            mov   edx, ebx
            neg   edx
          @ClampLoop:
            lodsd
            cmp   eax, ebx
{$IFDEF FIS_PENTIUM_PRO_OPTIMIZATION}
            cmovg eax, ebx
{$ELSE}
            jng   @NoClampHi
            mov   eax, ebx
          @NoClampHi:
{$ENDIF}
            cmp   eax, edx
{$IFDEF FIS_PENTIUM_PRO_OPTIMIZATION}
            cmovl eax, edx
{$ELSE}
            jnl   @NoClampLo
            mov   eax, edx
          @NoClampLo:
{$ENDIF}
            stosw
            loop  @ClampLoop

            popad
          end;

        end;

      Sleep( 50 );

    end;

end;


// ------------------------------------------------------------------------------------------------------------------------------
//    function fisMxmInitialize( _MxmData ): boolean;
// ------------------------------------------------------------------------------------------------------------------------------

function fisMxmInitialize( _MxmData: pointer ): boolean;
var
  _Result: MMRESULT;
begin

  fisMxmInitialize := false;

  // ----- Initialize MxmPlay.

  GetMem( fisMxmHeap, $4000 );
  asm
    pushad
    mov   edi, dword [ fisMxmHeap ]
    mov   esi, dword [ _MxmData ]
    call  xmpInit
    mov   dword [ _Result ], eax
    popad
  end;

  if ( _Result = 0 ) then
    Exit;

  // ----- Open WaveOut Device.

  _Result := waveOutOpen( @fisWaveOut, WAVE_MAPPER, @fisWaveFormatEx, 0, 0, CALLBACK_NULL );
  if ( _Result <> MMSYSERR_NOERROR ) then
    Exit;

  GetMem( fisMxmBuffer, FIS_WAVEOUT_BUFFERSIZE * 8 );
  if ( fisMxmBuffer = nil ) then
    Exit;

  GetMem( fisWaveOutBuffer, FIS_WAVEOUT_BUFFERSIZE * 4 );
  if ( fisWaveOutBuffer = nil ) then
    Exit;

  // ----- Prepare WaveOut.

  fisWaveHdr.lpData := pchar( fisWaveOutBuffer );
  _Result := waveOutPrepareHeader( fisWaveOut, @fisWaveHdr, sizeof( WAVEHDR ));
  if ( _Result <> MMSYSERR_NOERROR ) then
    Exit;

  // ----- Pre-render Audio Data.

  FillDword( fisWaveOutBuffer^, FIS_WAVEOUT_BUFFERSIZE, 0 );

  _Result := waveOutWrite( fisWaveOut, @fisWaveHdr, sizeof( WAVEHDR ));
  if ( _Result <> MMSYSERR_NOERROR ) then
    Exit;

  // ----- Create Playback Thread.

  fisMxmThreadExitRequest := false;
  fisMxmThread := CreateThread( nil, 0, @fisMxmThreadProc, 0, 0, fisMxmThreadID );
  if ( fisMxmThread = 0 ) then
    Exit;

  if ( SetThreadPriority( fisMxmThread, FIS_WAVEOUT_PRIORITY ) = false ) then
    Exit;

  fisMxmInitialize := true;

end;


// ------------------------------------------------------------------------------------------------------------------------------
//    procedure fisMxmPlay( _StartOrder );
// ------------------------------------------------------------------------------------------------------------------------------

procedure fisMxmPlay( _StartOrder: dword );
begin

  asm
    pushad
    mov   eax, dword [ _StartOrder ]
    call  xmpPlay
    popad
  end;

end;


// ------------------------------------------------------------------------------------------------------------------------------
//    procedure fisMxmSetVolume( _Volume );
// ------------------------------------------------------------------------------------------------------------------------------

procedure fisMxmSetVolume( _Volume: byte );
begin

  asm
    pushad
    mov   al, byte [ _Volume ]
    call  xmpSetVolume
    popad
  end;

end;


// ------------------------------------------------------------------------------------------------------------------------------
//    function fisMxmGetSync(): byte;
// ------------------------------------------------------------------------------------------------------------------------------

function fisMxmGetSync: byte;
var
  _Sync: byte;
begin

  asm
    pushad
    call  xmpGetSync
    mov   byte [ _Sync ], al
    popad
  end;

  fisMxmGetSync := _Sync;

end;


// ------------------------------------------------------------------------------------------------------------------------------
//    function fisMxmGetModulePosition(): word;
// ------------------------------------------------------------------------------------------------------------------------------

function fisMxmGetPos: word;
var
  _Position: word;
begin

  asm
    pushad
    call  xmpGetPos
    mov   word [ _Position ], ax
    popad
  end;

  fisMxmGetPos := _Position;

end;


// ------------------------------------------------------------------------------------------------------------------------------
//    function fisMxmGetWaveOutPosition(): dword;
// ------------------------------------------------------------------------------------------------------------------------------

function fisMxmGetWaveOutPosition: dword;
var
  _Result: MMRESULT;
begin

  fisMxmGetWaveOutPosition := 0;

  _Result := waveOutGetPosition( fisWaveOut, @fisMMTime, sizeof( MMTIME ));
  if ( _Result <> MMSYSERR_NOERROR ) then
    Exit;

  // ----- Adjust WaveOut Time (Latency).

  if ( fisMMTime.Sample > FIS_WAVEOUT_BUFFERSIZE ) then
    fisMxmGetWaveOutPosition := fisMMTime.Sample - FIS_WAVEOUT_BUFFERSIZE;

end;


// ------------------------------------------------------------------------------------------------------------------------------
//    procedure fisMxmRelease();
// ------------------------------------------------------------------------------------------------------------------------------

procedure fisMxmRelease();
begin

  waveOutReset( fisWaveOut );

  fisMxmThreadExitRequest := true;
  WaitForSingleObject( fisMxmThread, 1000 );
  fisMxmThread := 0;

  waveOutClose( fisWaveOut );
  fisWaveOut := 0;

end;
