
// ==============================================================================================================================
//    Fresh!mindworkz Intro Skeleton
// ------------------------------------------------------------------------------------------------------------------------------
//    Copyright (C) 2002. Pasy & Pontscho & Remage / Fresh!mindworkz.
// ==============================================================================================================================


const

  FIS_WINDOW_WIDTH       = 800;
  FIS_WINDOW_HEIGHT      = 600;
  FIS_WINDOW_COLORBITS   = 32;
  FIS_WINDOW_DEPTHBITS   = 24;
  FIS_WINDOW_FULLSCREEN  = true;


// ------------------------------------------------------------------------------------------------------------------------------
//    Initialized variables.
// ------------------------------------------------------------------------------------------------------------------------------

const

  fisWnd: HWND = 0;
  fisDC: HDC = 0;
  fisGLRC: HGLRC = 0;

  fisExitRequest: boolean = true;


// ------------------------------------------------------------------------------------------------------------------------------
//    procedure fisWindowDestroy();
// ------------------------------------------------------------------------------------------------------------------------------

procedure fisWindowDestroy(); inline;
begin

  if ( fisWnd <> 0 ) then
    begin

      if ( FIS_WINDOW_FULLSCREEN ) then
        ChangeDisplaySettings( 0, 0 );

      if ( fisDC <> 0 ) then
        begin

          if ( fisGLRC <> 0 ) then
            begin

              wglMakeCurrent( fisDC, 0 );
              wglDeleteContext( fisGLRC );
              fisGLRC := 0;

            end;

          ReleaseDC( fisDC, fisWnd );
          fisDC := 0;

        end;

      DestroyWindow( fisWnd );
      fisWnd := 0;

    end;

  fisExitRequest := true;

end;


// ------------------------------------------------------------------------------------------------------------------------------
//    function fisWindowProc( _hWnd, _Msg, _wParam, _lParam ): longint;
// ------------------------------------------------------------------------------------------------------------------------------

function fisWindowProc( _hWnd: HWND; _Msg, _wParam, _lParam: longint ): longint; stdcall; export;
begin

  case ( _Msg ) of

    WM_CHAR:
      begin

        // if ( _wParam = VK_SPACE ) then

        if ( _wParam = VK_ESCAPE ) then
          SendMessage( _hWnd, WM_CLOSE, 0, 0 );

      end;

    WM_CLOSE:
      fisWindowDestroy();

    WM_DESTROY:
      PostQuitMessage( 0 );

    else
      fisWindowProc := DefWindowProc( _hWnd, _Msg, _wParam, _lParam );

    end;

end;


// ------------------------------------------------------------------------------------------------------------------------------
//    Initialized variables.
// ------------------------------------------------------------------------------------------------------------------------------

const

  fisWndClass: WNDCLASS =
    (
      Style:          CS_OWNDC or CS_VREDRAW or CS_HREDRAW;
      lpfnWndProc:    WNDPROC( @fisWindowProc );
      cbClsExtra:     0;
      cbWndExtra:     0;
      hInstance:      0;
      hIcon:          0;
      hCursor:        0;
      hbrBackGround:  0;
      lpszMenuName:   nil;
      lpszClassName:  'fis';
    );

  fisDevMode: DEVMODE =
    (
      dmDeviceName:       ( 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 );
      dmSpecVersion:      0;
      dmDriverVersion:    0;
      dmSize:             sizeof( DEVMODE );
      dmDriverExtra:      0;
      dmFields:           DM_PELSWIDTH or DM_PELSHEIGHT or DM_BITSPERPEL;
      dmOrientation:      0;
      dmPaperSize:        0;
      dmPaperLength:      0;
      dmPaperWidth:       0;
      dmScale:            0;
      dmCopies:           0;
      dmDefaultSource:    0;
      dmPrintQuality:     0;
      dmColor:            0;
      dmDuplex:           0;
      dmYResolution:      0;
      dmTTOption:         0;
      dmCollate:          0;
      dmFormName:         ( 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 );
      dmLogPixels:        0;
      dmBitsPerPel:       FIS_WINDOW_COLORBITS;
      dmPelsWidth:        FIS_WINDOW_WIDTH;
      dmPelsHeight:       FIS_WINDOW_HEIGHT;
      dmDisplayFlags:     0;
      dmDisplayFrequency: 0;
    );

  fisPixelFormat: longint = 0;
  fisPixelFormatDescriptor: PIXELFORMATDESCRIPTOR =
    (
      nSize:            sizeof( PIXELFORMATDESCRIPTOR );
      nVersion:         1;
      dwFlags:          PFD_SUPPORT_OPENGL or PFD_DRAW_TO_WINDOW or PFD_DOUBLEBUFFER;
      iPixelType:       PFD_TYPE_RGBA;
      cColorBits:       FIS_WINDOW_COLORBITS;
      cRedBits:         0;
      cRedShift:        0;
      cGreenBits:       0;
      cGreenShift:      0;
      cBlueBits:        0;
      cBlueShift:       0;
      cAlphaBits:       0;
      cAlphaShift:      0;
      cAccumBits:       0;
      cAccumRedBits:    0;
      cAccumGreenBits:  0;
      cAccumBlueBits:   0;
      cAccumAlphaBits:  0;
      cDepthBits:       FIS_WINDOW_DEPTHBITS;
      cStencilBits:     0;
      cAuxBuffers:      0;
      iLayerType:       0;
      bReserved:        0;
      dwLayerMask:      0;
      dwVisibleMask:    0;
      dwDamageMask:     0;
    );


// ------------------------------------------------------------------------------------------------------------------------------
//    function fisWindowCreate(): HWND;
// ------------------------------------------------------------------------------------------------------------------------------

function fisWindowCreate(): HWND; inline;
begin

  fisWindowCreate := 0;

  // ----- Register window class.

  fisWndClass.hInstance := System.MainInstance;
  if ( RegisterClass( fisWndClass ) = 0 ) then
    Exit;

  // ----- Create & show window.

  fisWnd := CreateWindow( 'fis', 'autospace02', WS_VISIBLE or WS_POPUP, 0, 0, FIS_WINDOW_WIDTH, FIS_WINDOW_HEIGHT, 0, 0, System.MainInstance, nil );
  if ( fisWnd = 0 ) then
    Exit;

  ShowWindow( fisWnd, SW_SHOW );
  UpdateWindow( fisWnd );
  SetFocus( fisWnd );

  ShowCursor( false );

  // ----- Switch to fullscreen.

  if ( FIS_WINDOW_FULLSCREEN ) then
    ChangeDisplaySettings( fisDevMode, CDS_FULLSCREEN );

  // ----- Create OpenGL context.

  fisDC := GetDC( fisWnd );

  fisPixelFormat := ChoosePixelFormat( fisDC, fisPixelFormatDescriptor );
  if ( fisPixelFormat = 0 ) then
    Exit;

  if ( SetPixelFormat( fisDC, fisPixelFormat, @fisPixelFormatDescriptor ) = false ) then
    Exit;

  fisGLRC := wglCreateContext( fisDC );
  if ( fisGLRC = 0 ) then
    Exit;

  if ( wglMakeCurrent( fisDC, fisGLRC ) = false ) then
    Exit;

  fisExitRequest := false;
  fisWindowCreate := fisWnd;

end;


// ------------------------------------------------------------------------------------------------------------------------------
//    procedure fisWindowUpdate();
// ------------------------------------------------------------------------------------------------------------------------------

procedure fisWindowUpdate(); inline;
var
  _Msg: MSG;
begin

  SwapBuffers( fisDC );

  while ( PeekMessage( @_Msg, 0, 0, 0, PM_REMOVE )) do
    if ( _Msg.Message <> WM_QUIT ) then
      begin
        TranslateMessage( _Msg );
        DispatchMessage( _Msg );
      end
    else
      fisExitRequest := true;

end;
